﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace Time_Pilot.FlyingObjects
{
    class FlyingObject
    {
        protected int speed;
        protected int direction;
        protected Point position;
        public Boolean endCourse = false;       

        // default constuctor
        public FlyingObject()
        {
            speed = 0;
            direction = 0;
            position = new Point(0, 0);
        }

        // constructor with speed, direction and position
        public FlyingObject(int s, int d, Point p)
        {
            speed = s;
            direction = d;
            position = p;
        }

        // each flying object should draw itself on the (buffered) screen
        public virtual void Draw(SpriteBatch spriteBatch, Texture2D texture) {            
        }

        // set flying object speed
        public void setSpeed(int s)
        {
            speed = s;
        }

        // return flying object current speed
        public int getSpeed()
        {
            return speed;
        }

        // set the flying object direction
        public void setDirection(int d)
        {
            direction = d;
        }

        // get the flying object direction
        public int getDirection()
        {
            return direction;
        }

        // set the flying object position
        public void setPosition(Point p)
        {
            position = p;
        }

        public Point getPosition()
        {
            return position;
        }

        // turn ccw
        public virtual void turnCounterClockWise()
        {
            direction = (direction + 30) % 360;
        }

        // turn cw
        public virtual void turnClockWise()
        {
            if (direction == 0) direction = 330;
            else direction = (direction - 30) % 360;
        }


        // turn ccw from initial angle
        public void turnCounterClockWise(int d)
        {
            direction = (direction + d) % 360;
        }


        // turn cw from initial angle
        public void turnClockWise(int d)
        {
            if (direction == 0) direction = 330;
            else direction = (direction - d) % 360;
        }


        // convert degrees into radians
        public double rad(int d)
        {
            return Math.PI * d / 180;
        }


        // update flying object postion
        public virtual void step()
        {
            position.X += (int)(Math.Cos(rad(direction)) * (2 + speed));
            position.Y += (int)(Math.Sin(rad(direction)) * (2 + speed));
        }

        // return postion
        public Point pos()
        {
            return position;
        }

        // return X coordinate
        public int posX()
        {
            return position.X;
        }

        // return Y coordinate
        public int posY()
        {
            return position.Y;
        }


        // test if this object is hit by some bullets
        public virtual int isHitBy(ArrayList bullets)
        {
           
             ArrayList bulletsToRemove = new ArrayList();
             foreach (FlyingObject bullet in bullets)
             {

                 if ((Math.Abs((position.X - 19) - bullet.posX()) < 15) &&
                 (Math.Abs((position.Y - 19) - bullet.posY()) < 15))                    
                                      
                     bulletsToRemove.Add(bullet);                 
             }
             foreach (FlyingObject bullet in bulletsToRemove)
             {
                 bullets.Remove(bullet);
             }
             if (bulletsToRemove.Count > 0) return 0;
            
            
            
            if ((Math.Abs((position.X - 19) - 244) < 22) && (Math.Abs((position.Y - 19) - 195) < 22))
            {
                return 1;
            }
            else return 2;
        }


        // test if bullets have been fired in direction of this object
        public virtual Boolean bulletWillHit()
        {
            int x = position.X;
            int y = position.Y;
            for (int i = 0; i < 12; i++)
            {
                x += (int)(Math.Cos(rad(direction)) * 10);
                y += (int)(Math.Sin(rad(direction)) * 10);
                if ((x > 180 && x < 300) && (y > 135 && y < 275)) return true;
            }
            return false;
        }

        public virtual void fire()
        {
        }

        public virtual void dropBomb()
        {
        }
    }

    
}
