﻿
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using XNAExtras;
#endregion

namespace Time_Pilot
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class TimePilotGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;

        SpriteBatch spriteBatch;
        float elapsedCounter;

        // image files
        Texture2D sprites;
        Texture2D intro;

        // game screen
        TimePilotScreen screen;

        // decide wether game is started or not
        Boolean startGame = false;

        //Bitmap Font
        public static BitmapFont font;



        public TimePilotGame()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);

            this.TargetElapsedTime = new TimeSpan(300000);

            

            elapsedCounter = 0;

            screen = new TimePilotScreen(this);

        }

        public Texture2D getSprites()
        {
            return sprites;
        }

        public void stopThatGame()
        {
            startGame = false;
            GamePadState currentState = GamePad.GetState(PlayerIndex.One);
            if (currentState.IsConnected)
                GamePad.SetVibration(PlayerIndex.One, 0.0f, 0.0f);
        }



        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {            
            base.Initialize();
            graphics.PreferredBackBufferWidth = 527;
            graphics.PreferredBackBufferHeight = 428;
            graphics.ApplyChanges();
        }

        protected override void BeginRun()
        {           
            //Create the SpriteBatch object used for drawing the sprites
            spriteBatch = new SpriteBatch(this.graphics.GraphicsDevice);
            // initalize font
            font.Reset(graphics.GraphicsDevice, spriteBatch);
            font.KernEnable = true;

            //load game sounds
            SoundPlayer.Initialize();                        

            base.BeginRun();
        }



        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                sprites = content.Load<Texture2D>(@"Content\Images\sprites10");
                intro = content.Load<Texture2D>(@"Content\Images\intro");

                // load font
                font = new BitmapFont(@"Content\Fonts\courrier.xml");
            }
            // TODO: Load any ResourceManagementMode.Manual content                           
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // update sound engine
            SoundPlayer.Update();

            // The time since Update was called last
            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds; 


            //Keyboard Input 
            // TODO: change for GamePadHelper
            UpdateKeyboard(elapsed);

            UpdateController();
            if (startGame)
                screen.update(elapsed);
            // Let the GameComponents update

            base.Update(gameTime);
        }
        


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {                     
            graphics.GraphicsDevice.Clear(screen.getYear().color());
           
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend);
           
            if (startGame)
            {
                screen.Draw(spriteBatch);
            }
            else
            {
                spriteBatch.Draw(intro, new Rectangle(0, 0, 527, 428), Color.White);
            }
            spriteBatch.End();            
            base.Draw(gameTime); 
        }


        // get keyboard events from use
        // S: start
        // Space: fire
        // left: turn counter clock wise
        // right: turn clock wise
        protected void UpdateKeyboard(float elapsed)
        {
            KeyboardState aKeyboard = Keyboard.GetState();
            elapsedCounter += elapsed;
            if (aKeyboard.IsKeyDown(Keys.P) == true && !startGame)
            {
                startGame = true;
                screen = new TimePilotScreen(this);
            }
            if (aKeyboard.IsKeyDown(Keys.Space) == true && startGame)
            {
                screen.fire();
            }
            if (aKeyboard.IsKeyDown(Keys.Right) == true && startGame)
            {
                if (elapsedCounter > 0.05)
                {
                    screen.left();
                    elapsedCounter = 0;
                }
            }
            if (aKeyboard.IsKeyDown(Keys.Left) == true && startGame)
            {
                if (elapsedCounter > 0.05)
                {
                    screen.right();
                    elapsedCounter = 0;
                }
            }
        }

        //GamePadState m_PreviousGamePadState;
        //double prevAngle = 0;
        

        public void UpdateController()
        {
            GamePadState currentState = GamePad.GetState(PlayerIndex.One);
            
            if ((currentState.IsConnected)/* && (currentState.PacketNumber
                != m_PreviousGamePadState.PacketNumber)*/)
            {
                if (GamePad.GetState(PlayerIndex.One).Buttons.Start == ButtonState.Pressed && !startGame)
                {
                    startGame = true;
                    screen = new TimePilotScreen(this);
                }
                if ((GamePad.GetState(PlayerIndex.One).Buttons.LeftShoulder == ButtonState.Pressed && startGame) ||
                    (GamePad.GetState(PlayerIndex.One).Buttons.RightShoulder == ButtonState.Pressed && startGame)||
                    (GamePad.GetState(PlayerIndex.One).Triggers.Left > 0 && startGame)||
                    (GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed && startGame) ||
                    (GamePad.GetState(PlayerIndex.One).Triggers.Right > 0 && startGame))
                {
                    screen.fire();
                }

                if (GamePad.GetState(PlayerIndex.One).DPad.Left == ButtonState.Pressed) {
                    if (elapsedCounter > 0.05)
                    {
                        screen.right();
                        elapsedCounter = 0;
                    }
                }
                if (GamePad.GetState(PlayerIndex.One).DPad.Right == ButtonState.Pressed) {
                    if (elapsedCounter > 0.05)
                    {
                        screen.left();
                        elapsedCounter = 0;
                    }
                }

                // rotate time pilot plane
                /*
                Vector2 sp = GamePad.GetState(PlayerIndex.One).ThumbSticks.Right;
               
                double tanAlpha = sp.Y / sp.X;
                double angle = MathHelper.ToDegrees((float)Math.Atan(tanAlpha));

                if (sp.Y < 0 && sp.X >= 0)
                    angle = Math.Abs(angle);
                if (sp.Y < 0 && sp.X < 0)
                    angle = 180 - angle;
                if (sp.Y >= 0 && sp.X < 0)
                    angle = 180 + Math.Abs(angle);
                if (sp.Y > 0 && sp.X >= 0)
                    angle = 360 - angle;
                

                if (angle.Equals(Double.NaN)) angle = prevAngle;
                else prevAngle = angle;

                int targetAngle = 0;

                if (angle > 345 && angle <= 15)
                    targetAngle = 0;
                if (angle > 15 && angle <= 45)
                    targetAngle = 30;
                if (angle > 45 && angle <= 75)
                    targetAngle = 60;
                if (angle > 75 && angle <= 105)
                    targetAngle = 90;
                if (angle > 105 && angle <= 135)
                    targetAngle = 120;
                if (angle > 135 && angle <= 165)
                    targetAngle = 150;
                if (angle > 165 && angle <= 195)
                    targetAngle = 180;
                if (angle > 195 && angle <= 225)
                    targetAngle = 210;
                if (angle > 225 && angle <= 255)
                    targetAngle = 240;
                if (angle > 255 && angle <= 285)
                    targetAngle = 270;
                if (angle > 285 && angle <= 315)
                    targetAngle = 300;
                if (angle > 315 && angle <= 345)
                    targetAngle = 330;

                if (screen.timepilot.getDirection()!= targetAngle) screen.right();  
                 */

                // When finished with differences, update PreviousGamePadState.
                //m_PreviousGamePadState = currentState;

            }

        }

    }
}